//*****************************************************************************
// File Name	: lcdtest.c
// 
// Title		: example usage of some avr library functions
// Revision		: 1.0
// Notes		:	
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
// 
// Revision History:
// When			Who			Description of change
// -----------	-----------	-----------------------
// 30-Apr-2002	pstang		Created the program
//*****************************************************************************

//----- Include Files ---------------------------------------------------------
#include <avr/io.h>		// include I/O definitions (port names, pin names, etc)
#include <avr/interrupt.h>	// include interrupt support
#include <stdio.h> // include standard io

#include "global.h"		// include our global settings
//#include "uart.h"
//#include "rprintf.h"
#include "timer.h"		// include timer function library (timing, PWM, etc)
#include "spi.h"
#include "vt100.h"


#define MODE	PIN0
#define GSCLK	PIN1

#define XLAT	PIN2 //	(/SS of SPI If)
#define SCK		PIN5
#define MISO	PIN4
#define MOSI	PIN3

#define BLANK	PIN6
#define XERR	PIN7

void timer1CM(void);
void stopTimer1(void);
void setupTimer1(void);
void updateTimeout(void);

void startGSCLK(void);
void stopGSCLK(void);

void transferColorData(uint16_t nData);

static void uart_init(void);
static int uart_putchar(char c, FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);


//register uint16_t nGSCLKCycles asm("r2");
//register uint8_t nUpdateTimeout asm("r4");
volatile uint16_t nGSCLKCycles;
volatile uint8_t nUpdateTimeout;

volatile uint16_t nValue = 1;	// only 12bit used

volatile struct {
	unsigned fNeedUpdate:1;
	unsigned fUp:1;
} flags;


//----- Begin Code ------------------------------------------------------------


//----- Interrupt Handlers ------------------------------------------------------------
void timer1CM(void) {
	cbi(TIMSK, OCIE1A); // disable timer1 oc interrupt
	
	if (bit_is_clear(PINC, BLANK)) {
		//rprintfProgStrM("grayscale cycle running\r\n");
		if (nGSCLKCycles >= 4096) {
			//rprintfProgStrM("grayscale cycle completed\r\n");
			stopGSCLK();
			sbi(PORTC, BLANK);
			
			if (flags.fNeedUpdate == 1) {
				sbi(PORTB, XLAT);
				flags.fNeedUpdate = 0;
				//rprintfProgStrM("update done\r\n");
			}
			
			nUpdateTimeout = 8; // 1,02ms Timeout
			timerAttach(TIMER0OVERFLOW_INT, updateTimeout);
			TCNT0 = 0;
		} else {
			nGSCLKCycles++;
		}
	} else {
		if (flags.fNeedUpdate == 1) {
			sbi(PORTB, XLAT);
			flags.fNeedUpdate = 0;
			//rprintfProgStrM("update done\r\n");
		}
	}
	
	sbi(TIMSK, OCIE1A);
}

void updateTimeout(void) {
	if (nUpdateTimeout == 0) {
		timerDetach(TIMER0OVERFLOW_INT);
		cbi(PORTC, BLANK);
		startGSCLK();
	} else {
		nUpdateTimeout--;
	}
}

static void uart_init(void) {
	// initialize the USART (serial port)
	UBRRH = (uint8_t)UBRR_VAL >> 8;
	UBRRL = (uint8_t)UBRR_VAL & 0xFF;
	UCSRB = (1<<TXEN); //|(1<<TXCIE); // Enable Transmitter and Transmit interrupt
	UCSRC = (1<<URSEL)|(3<<UCSZ0); // Asynchron 8data, 1stop
}

static int uart_putchar(char c, FILE *stream) {
	if (c == '\n')
		uart_putchar('\r', stream);
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = c;
	return 0;
}


void stopTimer1(void) {
	timer1SetPrescaler(TIMER_CLK_STOP);
}
void setupTimer1(void) {
	// set timer1 to CTC Mode on OC1A
	cbi(TCCR1B, WGM13); sbi(TCCR1B, WGM12); // CTC Mode
	cbi(TCCR1A, WGM11); cbi(TCCR1A, WGM10); // CTC Mode
	// freq for timer1: F_CPU / 8 = 2 MHz
	TCNT1 = 0; // reset timer
	OCR1A = 15; // set GSCLK to 62,5 kHz
	
	sbi(TIMSK, OCIE1A); // enable oc1a interrupt
	nGSCLKCycles = 0;
	timerAttach(TIMER1OUTCOMPAREA_INT, timer1CM); // attach compare match interrupt
	
	timer1SetPrescaler(TIMER_CLK_DIV8); // start timer1 with F_CPU / 8
	
	startGSCLK();
}



void startGSCLK(void) {
	cbi(TCCR1A, COM1A1); sbi(TCCR1A, COM1A0); // toggle OC1A on compare match
	nGSCLKCycles = 0;
}

void stopGSCLK(void) {
	cbi(TCCR1A, COM1A1); cbi(TCCR1A, COM1A0); // normal port op
}

void transferColorData(uint16_t nData) {
	uint8_t n = 0;
	uint8_t nTransfer = 0;
	uint16_t nTransferMask = 0xFF0; // 111111110000b
	
	for (n = 0; n < 18; n++) {
		if (nTransferMask == 0xFF0) {
			nTransfer = (uint8_t)((nData & nTransferMask)>>4);
			nTransferMask = 0x00F;
		} else if (nTransferMask == 0x00F) {
			nTransfer = (uint8_t)(((nData & nTransferMask)<<4) | ((nData & 0xF00)>>8));
			nTransferMask = 0x0FF;
		} else {
			nTransfer = (uint8_t)(nData & nTransferMask);
			nTransferMask = 0xFF0;
		}
		spiTransferByte(nTransfer);
	}
}

int main(void)
{	
	// init flags
	flags.fNeedUpdate = 0;
	flags.fUp = 1;
	
	// init spi cs pin
	sbi(PORTB, XLAT); // enable pull-up, intermediate state
	sbi(DDRB, XLAT);
	
	// init other pins
	sbi(DDRB, MODE);
	cbi(PORTB, MODE);
	
	sbi(DDRB, GSCLK);
	cbi(PORTB, GSCLK);
	
	sbi(PORTC, BLANK); // enable pull-up, intermediate state
	sbi(DDRC, BLANK);
	sbi(PORTC, BLANK); // disable TLC5941 Outputs
	
	cbi(DDRC, XERR);
	cbi(PORTC, XERR); // disable pull-up, 100k hardware pull-ups
	
	uart_init();
	stdout = &mystdout;
	stderr = &mystdout;
		
	// initialize our libraries
	// initialize the UART (serial port)
	/*uartInit();
	// set the baud rate of the UART for our debug/reporting output
	uartSetBaudRate(9600);
	rprintfInit(uartSendByte);*/
	// initialize vt100 library
	vt100Init(&mystdout);
	
	// clear the terminal screen
	vt100ClearScreen();
	
	// initialize SPI
	spiInit();

	// initialize the timer system
	timerInit();
	timer0SetPrescaler(TIMER_CLK_DIV8);
	
	// configure timer
	//stopTimer1(); // stop timer1 for configuration
	setupTimer1();
	
	sei();
	
	// start operation
	cbi(PORTC, BLANK);
		
	//DDRD = 0xFF; // PORTD = output
	//uint16_t nValue = 1;	// only 12bit used

	
	//rprintfProgStrM("  init done\r\n");
	printf("init done\n");
	
	while(1)
	{
		//if (ClearToSend) { // generate new data for sending if the prev transfer is completed
		if (flags.fNeedUpdate == 0) { // if no update is scheduled send new data
			uint8_t n = 0;

			cbi(PORTB, MODE); // set Drivers to GS Mode
			cbi(PORTB, XLAT);		
			
			/* --- zero transfer for other drivers --- */
			for (n = 0; n < 48; n++) {
				spiTransferByte(0x00);
			}
			
			/* --- Color transfer --- */
			printf("sending value: %d\n", nValue);
			//rprintfNum(10, 10, FALSE, ' ', nValue);
			//rprintfProgStrM("\r\n");
			
			transferColorData(nValue);

			// change value
			if (nValue >= 4095) {
				flags.fUp = 0;
			} else if (nValue == 0) {
				flags.fUp = 1;
			}
			
			if (flags.fUp == 1) {
				nValue++;
			} else {
				nValue--;
			}
			
			/* --- White Transfer --- */
			for (n = 0; n < 6; n++) {
				spiTransferByte(0x00);
			}
			
			// schedule update
			printf("schedule update\n");
			flags.fNeedUpdate = 1;
		} else {
			if (bit_is_clear(PINC, BLANK)) {
				printf("Status: grayscale cycle running\n");
				if (nGSCLKCycles >= 4096) {
					printf("Status: grayscale cycle completed\n");
				}
			} else {
				printf("Status: grayscale cycle not running\n");
			}
		}
	}
	
	return 0;
}
